#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bitio.h"

#pragma warn -pia

BITFILE *bfopen(FILE *file) {
	BITFILE *bitfile = malloc(sizeof(BITFILE));
	if (!file) return NULL;
	if (bitfile) {
		bitfile->file = file;
		bitfile->name[0] = 0;
		bitfile->buf = 0;
		bitfile->n = 0;
	}
	return bitfile;
}

BITFILE *bopen(char *filename, char *mode) {
	BITFILE *result;
	if (result = bfopen(fopen(filename, mode))) strcpy(result->name, filename);
	return result;
}

int bclose(BITFILE *bitfile) {
	int i = fclose(bitfile->file);
	free(bitfile);
	return i;
}

int getbuf(BITFILE *bitfile) {
	bitfile->n = 8;
	return bitfile->buf = getc(bitfile->file);
}

void putbuf(BITFILE *bitfile) {
	if (bitfile->n) putc(bitfile->buf, bitfile->file);
	bitfile->buf = 0;
	bitfile->n = 0;
}

void clearbuf(BITFILE *bitfile) {
	bitfile->buf = 0;
	bitfile->n = 0;
}

int getb(BITFILE *bitfile) {
	if (!(bitfile->n)) getbuf(bitfile);
	bitfile->n--;
	return (bitfile->buf >> bitfile->n) & 1;
}

unsigned long getbits(int count, BITFILE *bitfile) {
	int i;
	unsigned long data;
	for (i=1, data=0; i <= count; i++)
		data |= (unsigned long) getb(bitfile) << (count-i);
	return data;
}

void putb(int b, BITFILE *bitfile) {
	if (bitfile->n == 8) putbuf(bitfile);
	bitfile->n++;
	bitfile->buf |= !!b << (8 - bitfile->n);
}

void putbits(unsigned long data, int count, BITFILE *bitfile) {
	int shift;
	while (count + bitfile->n >= 8) {
		shift = count - 8 + bitfile->n;
		bitfile->buf |= (data >> shift);
		putc(bitfile->buf, bitfile->file);
		count -= 8 - bitfile->n;
		data &= ~(data >> shift << shift);
		bitfile->buf = 0;
		bitfile->n = 0;
	}
	bitfile->n += count;
	bitfile->buf |= (data << (8 - bitfile->n));
}
